@@ -133,6 +133,8 @@ gnome_vfs_ssl_enabled (void)
 
 #endif
 
+#if defined(HAVE_OPENSSL) || defined(HAVE_GNUTLS)
+
 static GnomeVFSResult
 handle_ssl_read_write (int fd, int error, struct timeval *timeout,
 		       GnomeVFSCancellation *cancellation)
@@ -188,6 +190,8 @@ handle_ssl_read_write (int fd, int error, struct timeval *timeout,
 	return GNOME_VFS_ERROR_INTERNAL;
 }
 
+#endif
+
 /**
  * gnome_vfs_ssl_create:
  * @handle_return: pointer to a GnmoeVFSSSL struct, which will
@@ -396,7 +400,7 @@ gnome_vfs_ssl_create_from_fd (GnomeVFSSSL **handle_return,
 	res = GNOME_VFS_ERROR_IO;
 	err = gnutls_handshake (ssl->private->tlsstate);
 
-	if (ret == GNUTLS_E_INTERRUPTED || ret = GNUTLS_E_AGAIN) {
+	if (err == GNUTLS_E_INTERRUPTED || err == GNUTLS_E_AGAIN) {
 
 		res = handle_ssl_read_write (ssl->private->sockfd,
 					     gnutls_record_get_direction (ssl->private->tlsstate),
@@ -442,9 +446,9 @@ gnome_vfs_ssl_read (GnomeVFSSSL *ssl,
 		    GnomeVFSFileSize *bytes_read,
 		    GnomeVFSCancellation *cancellation)
 {
+#ifdef HAVE_OPENSSL
 	GnomeVFSResult res;
 	int ret;
-#ifdef HAVE_OPENSSL
 	int error;
 	
 	if (bytes == 0) {
@@ -483,13 +487,16 @@ gnome_vfs_ssl_read (GnomeVFSSSL *ssl,
 	
 	return GNOME_VFS_OK;
 #elif defined HAVE_GNUTLS
+	GnomeVFSResult res;
+	int ret;
+
 	if (bytes == 0) {
 		*bytes_read = 0;
 		return GNOME_VFS_OK;
 	}
 
- retry:
 	res = GNOME_VFS_OK;
+retry:
 	ret = gnutls_record_recv (ssl->private->tlsstate, buffer, bytes);
 
 	if (ret == 0) {
@@ -498,7 +505,7 @@ gnome_vfs_ssl_read (GnomeVFSSSL *ssl,
 
 		res = GNOME_VFS_ERROR_IO;
 
-		if (ret == GNUTLS_E_INTERRUPTED || ret = GNUTLS_E_AGAIN) {
+		if (ret == GNUTLS_E_INTERRUPTED || ret == GNUTLS_E_AGAIN) {
 			res = handle_ssl_read_write (ssl->private->sockfd,
 						     gnutls_record_get_direction (ssl->private->tlsstate),
 						     ssl->private->timeout,
@@ -540,9 +547,9 @@ gnome_vfs_ssl_write (GnomeVFSSSL *ssl,
 		     GnomeVFSFileSize *bytes_written,
 		     GnomeVFSCancellation *cancellation)
 {
+#ifdef HAVE_OPENSSL
 	GnomeVFSResult res;
 	int ret;
-#ifdef HAVE_OPENSSL
 	int error;
 	
 	if (bytes == 0) {
@@ -576,13 +583,16 @@ gnome_vfs_ssl_write (GnomeVFSSSL *ssl,
 	*bytes_written = ret;
 	return GNOME_VFS_OK;
 #elif defined HAVE_GNUTLS
+	GnomeVFSResult res;
+	int ret;
+
 	if (bytes == 0) {
 		*bytes_written = 0;
 		return GNOME_VFS_OK;
 	}
 
- retry:
 	res = GNOME_VFS_OK;
+retry:
 	ret = gnutls_record_send (ssl->private->tlsstate, buffer, bytes);
 
 	if (ret == 0) {
@@ -591,7 +601,7 @@ gnome_vfs_ssl_write (GnomeVFSSSL *ssl,
 
 		res = GNOME_VFS_ERROR_IO;
 
-		if (ret == GNUTLS_E_INTERRUPTED || ret = GNUTLS_E_AGAIN) {
+		if (ret == GNUTLS_E_INTERRUPTED || ret == GNUTLS_E_AGAIN) {
 			res = handle_ssl_read_write (ssl->private->sockfd,
 						     gnutls_record_get_direction (ssl->private->tlsstate),
 						     ssl->private->timeout,
@@ -624,9 +634,9 @@ void
 gnome_vfs_ssl_destroy (GnomeVFSSSL *ssl,
 		       GnomeVFSCancellation *cancellation) 
 {
+#ifdef HAVE_OPENSSL
 	int ret;
 	GnomeVFSResult res;	
-#ifdef HAVE_OPENSSL
 	int error;
 
  retry:
@@ -651,7 +661,9 @@ gnome_vfs_ssl_destroy (GnomeVFSSSL *ssl,
 	if (ssl->private->timeout)
 		g_free (ssl->private->timeout);
 #elif defined HAVE_GNUTLS
-
+	int ret;
+	GnomeVFSResult res;
+	
  retry:
 	ret = gnutls_bye (ssl->private->tlsstate, GNUTLS_SHUT_RDWR);
 	
