@@ -34,12 +34,17 @@
 #include "gnome-vfs-cdrom.h"
 #include "gnome-vfs-filesystem-type.h"
 
+#ifdef USE_HAL
+#include "gnome-vfs-hal-mounts.h"
+#endif /* USE_HAL */
+
 static void gnome_vfs_volume_monitor_daemon_class_init (GnomeVFSVolumeMonitorDaemonClass *klass);
 static void gnome_vfs_volume_monitor_daemon_init       (GnomeVFSVolumeMonitorDaemon      *volume_monitor_daemon);
 static void gnome_vfs_volume_monitor_daemon_finalize   (GObject                          *object);
 
 static void update_fstab_drives (GnomeVFSVolumeMonitorDaemon *volume_monitor_daemon);
 static void update_mtab_volumes (GnomeVFSVolumeMonitorDaemon *volume_monitor_daemon);
+
 static void update_connected_servers (GnomeVFSVolumeMonitorDaemon *volume_monitor_daemon);
 
 typedef struct {
@@ -51,6 +56,14 @@ typedef struct {
 
 static GnomeVFSVolumeMonitorClass *parent_class = NULL;
 
+/* In the event we can't connect to the HAL daemon this boolean is TRUE
+ * and we fall back to the usual fstab/mtab monitoring. 
+ *
+ * This is also useful for maintaing the non-HAL code on a system with
+ * HAL installed.
+ */
+static gboolean dont_use_hald = TRUE;
+
 /* Locking strategy:
  *
  * Its important that the public API is threadsafe, since it will be used
@@ -140,10 +153,22 @@ connected_servers_changed (GConfClient* client,
 static void
 gnome_vfs_volume_monitor_daemon_init (GnomeVFSVolumeMonitorDaemon *volume_monitor_daemon)
 {
-	_gnome_vfs_monitor_unix_mounts (fstab_changed,
-					volume_monitor_daemon,
-					mtab_changed,
-					volume_monitor_daemon);
+#ifdef USE_HAL
+	if (_gnome_vfs_monitor_hal_mounts_init (volume_monitor_daemon)) {
+		/* It worked, do use HAL */
+		dont_use_hald = FALSE;
+	} else {
+		/* Couldn't connect to HAL daemon, don't use HAL */
+		dont_use_hald = TRUE;
+	}
+#endif /* USE_HAL */
+
+	if (dont_use_hald) {
+		_gnome_vfs_monitor_unix_mounts (fstab_changed,
+						volume_monitor_daemon,
+						mtab_changed,
+						volume_monitor_daemon);
+	}
 
 	volume_monitor_daemon->gconf_client = gconf_client_get_default ();
 	gconf_client_add_dir (volume_monitor_daemon->gconf_client,
@@ -160,16 +185,35 @@ gnome_vfs_volume_monitor_daemon_init (GnomeVFSVolumeMonitorDaemon *volume_monito
 					 NULL);
 								       
 	
-	update_fstab_drives (volume_monitor_daemon);
-	update_mtab_volumes (volume_monitor_daemon);
+	if (dont_use_hald) {
+		update_fstab_drives (volume_monitor_daemon);
+		update_mtab_volumes (volume_monitor_daemon);
+	}
+#ifdef USE_HAL
+	else {
+		_gnome_vfs_monitor_hal_get_volume_list (volume_monitor_daemon);
+	}
+#endif /* USE_HAL */
+
 	update_connected_servers (volume_monitor_daemon);
 }
 
 void
-gnome_vfs_volume_monitor_daemon_force_probe (GnomeVFSVolumeMonitorDaemon *volume_monitor_daemon)
+gnome_vfs_volume_monitor_daemon_force_probe (GnomeVFSVolumeMonitor *volume_monitor)
 {
-	update_fstab_drives (volume_monitor_daemon);
-	update_mtab_volumes (volume_monitor_daemon);
+	GnomeVFSVolumeMonitorDaemon *volume_monitor_daemon;
+	
+	volume_monitor_daemon = GNOME_VFS_VOLUME_MONITOR_DAEMON (volume_monitor);
+	
+	if (dont_use_hald) {
+		update_fstab_drives (volume_monitor_daemon);
+		update_mtab_volumes (volume_monitor_daemon);
+	}
+#ifdef USE_HAL
+	else {
+		_gnome_vfs_monitor_hal_get_volume_list (volume_monitor_daemon);
+	}
+#endif /* USE_HAL */
 	update_connected_servers (volume_monitor_daemon);
 }
 
@@ -182,15 +226,22 @@ gnome_vfs_volume_monitor_daemon_finalize (GObject *object)
 
 	volume_monitor_daemon = GNOME_VFS_VOLUME_MONITOR_DAEMON (object);
 		
-	_gnome_vfs_stop_monitoring_unix_mounts ();
+	if (dont_use_hald) {
+		_gnome_vfs_stop_monitoring_unix_mounts ();
 
-	g_list_foreach (volume_monitor_daemon->last_mtab,
-			(GFunc)_gnome_vfs_unix_mount_free, NULL);
-	g_list_free (volume_monitor_daemon->last_mtab);
+		g_list_foreach (volume_monitor_daemon->last_mtab,
+				(GFunc)_gnome_vfs_unix_mount_free, NULL);
+		g_list_free (volume_monitor_daemon->last_mtab);
 	
-	g_list_foreach (volume_monitor_daemon->last_fstab,
-			(GFunc)_gnome_vfs_unix_mount_point_free, NULL);
-	g_list_free (volume_monitor_daemon->last_fstab);	
+		g_list_foreach (volume_monitor_daemon->last_fstab,
+				(GFunc)_gnome_vfs_unix_mount_point_free, NULL);
+		g_list_free (volume_monitor_daemon->last_fstab);	
+	}
+#ifdef USE_HAL
+	else {
+		_gnome_vfs_monitor_hal_mounts_shutdown (volume_monitor_daemon);
+	}
+#endif /* USE_HAL */
 
 	gconf_client_notify_remove (volume_monitor_daemon->gconf_client,
 				    volume_monitor_daemon->connected_id);
@@ -241,9 +292,28 @@ get_device_type_from_device_and_mount (const char *device_path, const char *moun
 			}
 	} else {
 		basename = g_path_get_basename (mount_path);
+
 		if (g_str_has_prefix (basename, "cdrom") ||
 		    g_str_has_prefix (basename, "cdwriter") ||
-		    g_str_has_prefix (basename, "burn")) {
+		    g_str_has_prefix (basename, "burn") ||
+		    g_str_has_prefix (basename, "cdr") ||
+		    g_str_has_prefix (basename, "cdrw") ||
+		    g_str_has_prefix (basename, "dvdrom") ||
+		    g_str_has_prefix (basename, "dvdram") ||
+		    g_str_has_prefix (basename, "dvdr") ||
+		    g_str_has_prefix (basename, "dvdrw") ||
+		    g_str_has_prefix (basename, "cdrom_dvdrom") ||
+		    g_str_has_prefix (basename, "cdrom_dvdram") ||
+		    g_str_has_prefix (basename, "cdrom_dvdr") ||
+		    g_str_has_prefix (basename, "cdrom_dvdrw") ||
+		    g_str_has_prefix (basename, "cdr_dvdrom") ||
+		    g_str_has_prefix (basename, "cdr_dvdram") ||
+		    g_str_has_prefix (basename, "cdr_dvdr") ||
+		    g_str_has_prefix (basename, "cdr_dvdrw") ||
+		    g_str_has_prefix (basename, "cdrw_dvdrom") ||
+		    g_str_has_prefix (basename, "cdrw_dvdram") ||
+		    g_str_has_prefix (basename, "cdrw_dvdr") ||
+		    g_str_has_prefix (basename, "cdrw_dvdrw")) {
 			g_free (basename);
 			return GNOME_VFS_DEVICE_TYPE_CDROM;
 		} else if (g_str_has_prefix (basename, "floppy")) {
@@ -259,6 +329,10 @@ get_device_type_from_device_and_mount (const char *device_path, const char *moun
 			g_free (basename);
 			return GNOME_VFS_DEVICE_TYPE_CAMERA;
 		} else if (g_str_has_prefix (basename, "memstick") ||
+			   g_str_has_prefix (basename, "compact_flash") ||
+			   g_str_has_prefix (basename, "memory_stick") ||
+			   g_str_has_prefix (basename, "smart_media") ||
+			   g_str_has_prefix (basename, "sd_mmc") ||
 			   g_str_has_prefix (basename, "ram")) {
 			g_free (basename);
 			return GNOME_VFS_DEVICE_TYPE_MEMORY_STICK;
@@ -321,7 +395,7 @@ make_utf8 (const char *str)
 }
 
 static char *
-get_drive_icon_from_type (GnomeVFSDeviceType type)
+get_drive_icon_from_type (GnomeVFSDeviceType type, const char *mount_path)
 {
 	char *icon_name;
 
@@ -334,6 +408,22 @@ get_drive_icon_from_type (GnomeVFSDeviceType type)
 	case GNOME_VFS_DEVICE_TYPE_SMB:
 		icon_name = "gnome-fs-smb";
 		break;
+
+	case GNOME_VFS_DEVICE_TYPE_HARDDRIVE:
+		if (mount_path != NULL) {
+			char *basename;	
+			basename = g_path_get_basename (mount_path);
+			if (g_str_has_prefix (basename, 
+					      "usbdisk")) {
+				icon_name = "gnome-dev-removable-usb";
+			} else if (g_str_has_prefix (basename, 
+						     "ieee1394disk")) {
+				icon_name = "gnome-dev-removable-1394";
+			}
+			g_free (basename);
+		}
+		break;
+
 	default:
 		break;
 	}
@@ -342,12 +432,31 @@ get_drive_icon_from_type (GnomeVFSDeviceType type)
 }
 
 static char *
-get_icon_from_type (GnomeVFSDeviceType type)
+get_icon_from_type (GnomeVFSDeviceType type, const char *mount_path)
 {
 	char *icon_name;
+	char *basename;	
+
+	if (mount_path != NULL) {
+		basename = g_path_get_basename (mount_path);
+	} else {
+		basename = NULL;
+	}
 
 	icon_name = "gnome-dev-harddisk";
 	switch (type) {
+	case GNOME_VFS_DEVICE_TYPE_HARDDRIVE:
+		if (basename != NULL) {
+			if (g_str_has_prefix (basename, 
+					      "usbdisk")) {
+				icon_name = "gnome-dev-harddisk-usb";
+			} else if (g_str_has_prefix (basename, 
+						     "ieee1394disk")) {
+				icon_name = "gnome-dev-harddisk-1394";
+			}
+		}
+		break;
+
 	case GNOME_VFS_DEVICE_TYPE_AUDIO_CD:
 		icon_name = "gnome-dev-cdrom-audio";
 		break;
@@ -369,6 +478,21 @@ get_icon_from_type (GnomeVFSDeviceType type)
 
 	case GNOME_VFS_DEVICE_TYPE_MEMORY_STICK:
 		icon_name = "gnome-dev-memory";
+		if (basename != NULL) {
+			if (g_str_has_prefix (basename, 
+					      "compact_flash")) {
+				icon_name = "gnome-dev-media-cf";
+			} else if (g_str_has_prefix (basename, 
+						     "memory_stick")) {
+				icon_name = "gnome-dev-media-ms";
+			} else if (g_str_has_prefix (basename, 
+						     "smart_media")) {
+				icon_name = "gnome-dev-media-sm";
+			} else if (g_str_has_prefix (basename, 
+						     "sd_mmc")) {
+				icon_name = "gnome-dev-media-sdmmc";
+			}
+		}
 		break;
 	
 	case GNOME_VFS_DEVICE_TYPE_NFS:
@@ -383,9 +507,14 @@ get_icon_from_type (GnomeVFSDeviceType type)
 		break;
 	}
 
+	if (basename != NULL) {
+		g_free (basename);
+	}
+
 	return g_strdup (icon_name);
 }
 
+
 static void
 diff_sorted_lists (GList *list1, GList *list2, GCompareFunc compare,
 		   GList **added, GList **removed)
@@ -443,6 +572,34 @@ get_drive_name (GnomeVFSVolumeMonitor *volume_monitor,
 	struct { char *machine; char *readable; } readable_table[] = {
 		{ "floppy", N_("Floppy") },
 		{ "cdrom", N_("CD-ROM") },
+
+		{ "cdr",    N_("CD-R") },
+		{ "cdrw",   N_("CD-RW") },
+		{ "dvdrom", N_("DVD-ROM") },
+		{ "dvdram", N_("DVD-RAM") },
+		{ "dvdr",   N_("DVD-R") },
+		{ "dvdrw",  N_("DVD-RW") },
+		{ "cdrom_dvdrom", N_("CD-ROM/DVD-ROM") },
+		{ "cdrom_dvdram", N_("CD-ROM/DVD-RAM") },
+		{ "cdrom_dvdr",   N_("CD-ROM/DVD-R") },
+		{ "cdrom_dvdrw",  N_("CD-ROM/DVD-RW") },
+		{ "cdr_dvdrom", N_("CD-R/DVD-ROM") },
+		{ "cdr_dvdram", N_("CD-R/DVD-RAM") },
+		{ "cdr_dvdr",   N_("CD-R/DVD-R") },
+		{ "cdr_dvdrw",  N_("CD-R/DVD-RW") },
+		{ "cdrw_dvdrom", N_("CD-RW/DVD-ROM") },
+		{ "cdrw_dvdram", N_("CD-RW/DVD-RAM") },
+		{ "cdrw_dvdr",   N_("CD-RW/DVD-R") },
+		{ "cdrw_dvdrw",  N_("CD-RW/DVD-RW") },
+
+		{ "idedisk", N_("Disk") },
+		{ "usbdisk", N_("USB Drive") },
+		{ "ieee1394disk", N_("IEEE1394 Drive") },
+		{ "compact_flash", N_("CF") },
+		{ "sd_mmc", N_("SD/MMC") },
+		{ "memory_stick", N_("Memory Stick") },
+		{ "smart_media", N_("Smart Media") },
+
 		{ "zip", N_("Zip Drive") },
 		{ "memstick", N_("Memory Stick") },
 		{ "camera", N_("Camera") },
@@ -539,12 +696,12 @@ create_drive_from_mount_point (GnomeVFSVolumeMonitor *volume_monitor,
 		}
 	}
 
-	drive->priv->icon = get_drive_icon_from_type (drive->priv->device_type);
+	drive->priv->icon = get_drive_icon_from_type (drive->priv->device_type, mount->mount_path);
 
 	drive->priv->display_name = get_drive_name (volume_monitor, drive, mount);
 	
 	drive->priv->is_user_visible = TRUE;
-	drive->priv->volume = NULL;
+	drive->priv->volumes = NULL;
 
 	uri = gnome_vfs_get_uri_from_local_path (mount->mount_path);
 	mounted_volume = _gnome_vfs_volume_monitor_find_mtab_volume_by_activation_uri (volume_monitor, uri);
@@ -552,7 +709,7 @@ create_drive_from_mount_point (GnomeVFSVolumeMonitor *volume_monitor,
 
 	if (mounted_volume != NULL &&
 	    mounted_volume->priv->drive == NULL) {
-		drive->priv->volume = gnome_vfs_volume_ref (mounted_volume);
+		_gnome_vfs_drive_add_mounted_volume (drive, mounted_volume);
 		_gnome_vfs_volume_set_drive (mounted_volume, drive);
 	}
 
@@ -694,7 +851,7 @@ create_vol_from_mount (GnomeVFSVolumeMonitor *volume_monitor, GnomeVFSUnixMount
 
 	vol->priv->volume_type = GNOME_VFS_VOLUME_TYPE_MOUNTPOINT;
 	vol->priv->device_path = g_strdup (mount->device_path);
-	vol->priv->unix_device = _gnome_vfs_unix_mount_get_unix_device (mount);
+	vol->priv->unix_device = 0;  /* Caller must fill in.  */
 	vol->priv->activation_uri = gnome_vfs_get_uri_from_local_path (mount->mount_path);
 	vol->priv->filesystem_type = g_strdup (mount->filesystem_type);
 	vol->priv->is_read_only = mount->is_read_only;
@@ -753,9 +910,11 @@ create_vol_from_mount (GnomeVFSVolumeMonitor *volume_monitor, GnomeVFSUnixMount
 		case CDS_XA_2_2:
 		case CDS_MIXED:
 			/* Get volume name */
-			tmp_name = _gnome_vfs_get_iso9660_volume_name (fd);
-			display_name = modify_volume_name_for_display (tmp_name);
-			g_free (tmp_name);
+			if (fd > 0) {
+				tmp_name = _gnome_vfs_get_iso9660_volume_name (fd);
+				display_name = modify_volume_name_for_display (tmp_name);
+				g_free (tmp_name);
+			}
 			break;
 			
 		default:
@@ -782,7 +941,7 @@ create_vol_from_mount (GnomeVFSVolumeMonitor *volume_monitor, GnomeVFSUnixMount
 	g_free (display_name);
 	g_free (utf8_name);
 	
-	vol->priv->icon = get_icon_from_type (vol->priv->device_type);
+	vol->priv->icon = get_icon_from_type (vol->priv->device_type, mount->mount_path);
 
 	vol->priv->is_user_visible = 0;
 	switch (vol->priv->device_type) {
@@ -805,14 +964,14 @@ create_vol_from_mount (GnomeVFSVolumeMonitor *volume_monitor, GnomeVFSUnixMount
 	g_free (uri);
 
 	if (containing_drive != NULL &&
-	    containing_drive->priv->volume == NULL) {
+	    g_list_find (containing_drive->priv->volumes, vol) == NULL) {
 		/* Make sure the mounted volume for a visible drive is visible */
 		if (containing_drive->priv->is_user_visible) {
 			vol->priv->is_user_visible = 1;
 		}
 		
 		vol->priv->drive = containing_drive;
-		_gnome_vfs_drive_set_mounted_volume (containing_drive, vol);
+		_gnome_vfs_drive_add_mounted_volume (containing_drive, vol);
 	}
 
 	return vol;
@@ -822,9 +981,8 @@ static void
 update_mtab_volumes (GnomeVFSVolumeMonitorDaemon *volume_monitor_daemon)
 {
 	GList *new_mtab;
-	GList *removed, *added;
-	GList *l;
-	GnomeVFSUnixMount *mount;
+	GList *removed, *added, *devices;
+	GList *l, *ld;
 	char *uri;
 	GnomeVFSVolume *vol;
 	GnomeVFSVolumeMonitor *volume_monitor;
@@ -839,7 +997,7 @@ update_mtab_volumes (GnomeVFSVolumeMonitorDaemon *volume_monitor_daemon)
 				   &added, &removed);
 
 		for (l = removed; l != NULL; l = l->next) {
-			mount = l->data;
+			GnomeVFSUnixMount *mount = l->data;
 			uri = gnome_vfs_get_uri_from_local_path (mount->mount_path);
 
 			vol = _gnome_vfs_volume_monitor_find_mtab_volume_by_activation_uri (volume_monitor, uri);
@@ -852,15 +1010,22 @@ update_mtab_volumes (GnomeVFSVolumeMonitorDaemon *volume_monitor_daemon)
 
 			g_free (uri);
 		}
-		
-		for (l = added; l != NULL; l = l->next) {
-			mount = l->data;
+
+		devices = _gnome_vfs_unix_mount_get_unix_device (added);
+
+		for (l = added, ld = devices;
+		     l != NULL;
+		     l = l->next, ld = ld->next) {
+			GnomeVFSUnixMount *mount = l->data;
+			dev_t unix_device = GPOINTER_TO_UINT (ld->data);
 		
 			vol = create_vol_from_mount (volume_monitor, mount);
+			vol->priv->unix_device = unix_device;
 			_gnome_vfs_volume_monitor_mounted (volume_monitor, vol);
 			gnome_vfs_volume_unref (vol);
 		}
-		
+
+		g_list_free (devices);
 		g_list_free (added);
 		g_list_free (removed);
 		g_list_foreach (volume_monitor_daemon->last_mtab,
