@@ -1101,25 +1101,17 @@ append_trash_path (const char *path)
 	}
 }
 
-/* Try to find the Trash in @current_directory. If not found, collect all the 
- * directories in @current_directory to visit later.
- */
 static char *
-find_trash_in_one_hierarchy_level (const char *current_directory, dev_t near_device_id, 
-	GList **directory_list, GnomeVFSContext *context)
+find_trash_in_hierarchy (const char *start_dir, dev_t near_device_id, GnomeVFSContext *context)
 {
 	char *trash_path;
-	char *item_path;
 	struct stat stat_buffer;
-	DIR *directory;
-	struct dirent *item_buffer;
-	struct dirent *item;
 
 	if (gnome_vfs_context_check_cancellation (context))
 		return NULL;
 
 	/* check if there is a trash in this directory */
-	trash_path = append_trash_path (current_directory);
+	trash_path = append_trash_path (start_dir);
 	if (lstat (trash_path, &stat_buffer) == 0 && S_ISDIR (stat_buffer.st_mode)) {
 		/* found it, we are done */
 		g_assert (near_device_id == stat_buffer.st_dev);
@@ -1127,102 +1119,9 @@ find_trash_in_one_hierarchy_level (const char *current_directory, dev_t near_dev
 	}
 	g_free (trash_path);
 
-
-	if (gnome_vfs_context_check_cancellation (context))
-		return NULL;
-
-	/* Trash not in this directory.
-	 * Collect the list of all the directories in this directory to visit later.
-	 */
-	directory = opendir (current_directory);
-	if (directory == NULL) {
-		return NULL;
-	}
-
-	item_buffer = g_malloc (sizeof (struct dirent) + GET_PATH_MAX() + 1);
-	for (;;) {
-#ifdef HAVE_READDIR_R
-		if (readdir_r (directory, item_buffer, &item) != 0 || item == NULL) {
-			break;
-		}
-#else
-		G_LOCK (readdir);
-		item = readdir (directory);
-		if (item == NULL) {
-			G_UNLOCK (readdir);
-			break;
-		}
-#endif
-
-		if (gnome_vfs_context_check_cancellation (context)) {
-#ifndef HAVE_READDIR_R
-			G_UNLOCK (readdir);
-#endif
-			break;
-		}
-
-		if (strcmp (item->d_name, ".") == 0
-			|| strcmp (item->d_name, "..") == 0) {
-#ifndef HAVE_READDIR_R
-			G_UNLOCK (readdir);
-#endif
-			continue;
-		}
-
-		item_path = append_to_path (current_directory, item->d_name);
-#ifndef HAVE_READDIR_R
-		G_UNLOCK (readdir);
-#endif
-		if (lstat (item_path, &stat_buffer) == 0 
-			&& S_ISDIR (stat_buffer.st_mode)
-			&& near_device_id == stat_buffer.st_dev) {
-
-			/* Directory -- put it on the list to search, 
-			 * just as long as it is on the same device.
-			 */
-			*directory_list = g_list_prepend (*directory_list, item_path);
-		} else {
-			g_free (item_path);
-		}
-		if (gnome_vfs_context_check_cancellation (context))
-			break;
-	}
-
-
-	closedir (directory);
-	g_free (item_buffer);
 	return NULL;
 }
 
-/* Do a width-first search of the directory hierarchy starting at start_dir,
- * looking for the trash directory. 
- * Not doing a traditional depth-first search here to prevent descending too deep in
- * the hierarchy -- we expect the Trash to be in a reasonably "shallow" location.
- * 
- * We only look MAX_TRASH_SEARCH_DEPTH deep, if the Trash is deeper in the hierarchy,
- * we will fail to find it.
- */
-static char *
-find_trash_in_hierarchy (const char *start_dir, dev_t near_device_id, GnomeVFSContext *context)
-{
-	GList *next_directory_list;
-	char *result;
-
-#ifdef DEBUG_FIND_DIRECTORY
-	g_print ("searching for trash in %s\n", start_dir);
-#endif
-
-	next_directory_list = NULL;
-
-	/* Search the top level. */
-	result = find_trash_in_one_hierarchy_level (start_dir, near_device_id, 
-		&next_directory_list, context);
-	
-	gnome_vfs_list_deep_free (next_directory_list);
-
-	return result;
-}
-
 static GList *cached_trash_directories;
 G_LOCK_DEFINE_STATIC (cached_trash_directories);
 
@@ -1295,7 +1194,7 @@ find_disk_top_directory (const char *item_on_disk,
 		}
 		
 		*last_slash = '\0';
-		if (lstat (disk_top_directory, &stat_buffer) < 0
+		if (stat (disk_top_directory, &stat_buffer) < 0
 			|| stat_buffer.st_dev != near_device_id) {
 			/* we ran past the root of the disk we are exploring */
 			g_free (disk_top_directory);
@@ -1461,6 +1360,7 @@ read_saved_cached_trash_entries (void)
 	char escaped_mount_point[PATH_MAX], escaped_trash_path[PATH_MAX];
 	char *mount_point, *trash_path;
 	struct stat stat_buffer;
+	gboolean removed_item;
 
 	/* empty the old locally cached entries */
 	g_list_foreach (cached_trash_directories, 
@@ -1474,6 +1374,7 @@ read_saved_cached_trash_entries (void)
 	cache_file = fopen (cache_file_path, "r");
 
 	if (cache_file != NULL) {
+		removed_item = FALSE;
 		for (;;) {
 			if (fgets (buffer, sizeof (buffer), cache_file) == NULL) {
 				break;
@@ -1488,22 +1389,31 @@ read_saved_cached_trash_entries (void)
 
 				if (trash_path != NULL 
 					&& mount_point != NULL
-					&& (strcmp (trash_path, NON_EXISTENT_TRASH_ENTRY) == 0 || lstat (trash_path, &stat_buffer) == 0)
-					&& lstat (mount_point, &stat_buffer) == 0) {
-					/* We either know the trash doesn't exist or we checked that it's really
+					&& (strcmp (trash_path, NON_EXISTENT_TRASH_ENTRY) != 0 && lstat (trash_path, &stat_buffer) == 0)
+					&& stat (mount_point, &stat_buffer) == 0) {
+					/* We know the trash exist and we checked that it's really
 					 * there - this is a good entry, copy it into the local cache.
+					 * We don't want to rely on old non-existing trash entries, as they
+					 * could have changed now, and they stick around filling up the cache,
+					 * and slowing down startup.
 					 */
 					 add_local_cached_trash_entry (stat_buffer.st_dev, trash_path, mount_point);
 #ifdef DEBUG_FIND_DIRECTORY
 					g_print ("read trash item cache entry %s %s\n", trash_path, mount_point);
 #endif
+				} else {
+					removed_item = TRUE;
 				}
 			}
 			
 			g_free (trash_path);
 			g_free (mount_point);
 		}
-		fclose (cache_file);	
+		fclose (cache_file);
+		/* Save cache to get rid of stuff from on-disk cache */
+		if (removed_item) {
+			save_trash_entry_cache ();
+		}
 	}
 	
 	g_free (cache_file_path);
@@ -1604,7 +1514,7 @@ find_or_create_trash_near (const char *full_name_near, dev_t near_device_id,
 	result = NULL;
 	/* figure out the topmost disk directory */
 	disk_top_directory = find_disk_top_directory (full_name_near, 
-		near_device_id, context);
+						      near_device_id, context);
 
 	if (disk_top_directory == NULL) {
 		/* Failed to find it, don't look at this disk until we
