@@ -84,9 +84,9 @@ entry_unref (Entry *entry)
 	entry->refcnt--;
 
 	if (entry->refcnt == 0) {
-		g_print ("-- KILLING ENTRY: (%p) %s---\n",
-			 entry,
-			 entry->displayname);
+		D (g_print ("-- KILLING ENTRY: (%p) %s---\n",
+			    entry,
+			    entry->displayname));
 
 		vfolder_info_remove_entry (entry->info, entry);
 
@@ -258,9 +258,11 @@ entry_reload_if_needed (Entry *entry)
 							 parsed[i]);
 			if (dep) {
 				vfolder_info_remove_entry (entry->info, dep);
+#if 0 /* vfolder_monitor_emit is not defined */
 				vfolder_monitor_emit (
 					entry_get_filename (dep),
 					GNOME_VFS_MONITOR_EVENT_DELETED);
+#endif
 				entry_unref (dep);
 			}
 		}
@@ -423,19 +425,19 @@ entry_dump (Entry *entry, int indent)
 	gchar *space = g_strnfill (indent, ' ');
 	GSList *keywords = entry->keywords, *iter;
 
-	g_print ("%s%s\n%s  Filename: %s\n%s  Keywords: ",
-		 space,
-		 entry_get_displayname (entry),
-		 space,
-		 entry_get_filename (entry),
-		 space);
+	D (g_print ("%s%s\n%s  Filename: %s\n%s  Keywords: ",
+		    space,
+		    entry_get_displayname (entry),
+		    space,
+		    entry_get_filename (entry),
+		    space));
 
 	for (iter = keywords; iter; iter = iter->next) {
-		GQuark quark = GPOINTER_TO_INT (iter->data);
-		g_print (g_quark_to_string (quark));
+		G_GNUC_UNUSED GQuark quark = GPOINTER_TO_INT (iter->data);
+		D (g_print (g_quark_to_string (quark)));
 	}
 
-	g_print ("\n");
+	D (g_print ("\n"));
 
 	g_free (space);
 }
@@ -523,8 +525,7 @@ folder_unref (Folder *folder)
 			g_hash_table_destroy (folder->excludes);
 		}
 
-		if (folder->includes_ht)
-			g_hash_table_destroy (folder->includes_ht);
+		g_slist_foreach (folder->includes, (GFunc) g_free, NULL);
 		g_slist_free (folder->includes);
 
 		/* subfolders */
@@ -564,12 +565,12 @@ folder_extend_monitor_cb (GnomeVFSMonitorHandle    *handle,
 	if (!strcmp (monitor_uri, info_uri))
 		return;
 
-	g_print ("*** Exdended folder %s ('%s') monitor %s%s%s called! ***\n",
-		 folder->name,
-		 info_uri,
-		 event_type == GNOME_VFS_MONITOR_EVENT_CREATED ? "CREATED":"",
-		 event_type == GNOME_VFS_MONITOR_EVENT_DELETED ? "DELETED":"",
-		 event_type == GNOME_VFS_MONITOR_EVENT_CHANGED ? "CHANGED":"");
+	D (g_print ("*** Exdended folder %s ('%s') monitor %s%s%s called! ***\n",
+		    folder->name,
+		    info_uri,
+		    event_type == GNOME_VFS_MONITOR_EVENT_CREATED ? "CREATED":"",
+		    event_type == GNOME_VFS_MONITOR_EVENT_DELETED ? "DELETED":"",
+		    event_type == GNOME_VFS_MONITOR_EVENT_CHANGED ? "CHANGED":""));
 
 	uri = gnome_vfs_uri_new (info_uri);
 	filename = gnome_vfs_uri_extract_short_name (uri);
@@ -1040,8 +1041,8 @@ folder_reload_if_needed (Folder *folder)
 	if (!folder->dirty || folder->loading)
 		return;
 
-	g_print ("----- RELOADING FOLDER: %s -----\n",
-		 folder->name);
+	D (g_print ("----- RELOADING FOLDER: %s -----\n",
+		    folder->name));
 
 	folder->loading = TRUE;
 	folder->info->loading = TRUE;
@@ -1365,19 +1366,10 @@ folder_add_entry (Folder *folder, Entry *entry)
 void
 folder_add_include (Folder *folder, const gchar *include)
 {
-	char *str = g_strdup (include);
-	
 	folder_remove_exclude (folder, include);
 	
-	if (!folder->includes_ht)
-		folder->includes_ht = 
-			g_hash_table_new_full (g_str_hash, 
-					       g_str_equal,
-					       (GDestroyNotify) g_free,
-					       NULL);
-
-	folder->includes = g_slist_prepend (folder->includes, str);
-	g_hash_table_insert (folder->includes_ht, str, folder->includes);
+	folder->includes = g_slist_prepend (folder->includes, 
+					    g_strdup (include));
 
 	vfolder_info_set_dirty (folder->info);
 }
@@ -1387,21 +1379,16 @@ folder_remove_include (Folder *folder, const gchar *file)
 {
 	GSList *li;
 
-	if (!folder->includes_ht)
+	if (!folder->includes)
 		return;
 
-	li = g_hash_table_lookup (folder->includes_ht, file);
+	li = g_slist_find_custom (folder->includes, 
+				  file, 
+				  (GCompareFunc) strcmp);
 	if (li) {
-		g_free (li->data);
-		/* 
-		 * Note: this will NOT change
-		 * folder->includes pointer! 
-		 */
 		folder->includes = g_slist_delete_link (folder->includes, li);
-		g_hash_table_remove (folder->includes_ht, li);
+		vfolder_info_set_dirty (folder->info);
 	}
-
-	vfolder_info_set_dirty (folder->info);
 }
 
 void
@@ -1507,10 +1494,10 @@ folder_dump_tree (Folder *folder, int indent)
 	const GSList *iter;
 	gchar *space = g_strnfill (indent, ' ');
 
-	g_print ("%s(%p): %s\n",
-		 space,
-		 folder,
-		 folder ? folder_get_name (folder) : NULL);
+	D (g_print ("%s(%p): %s\n",
+		    space,
+		    folder,
+		    folder ? folder_get_name (folder) : NULL));
 
 	g_free (space);
 
